import numpy as np


def droplets(shape, spacing=15, radius=3, mix=.3):
    """Return a 2D texture array of spherical droplets.
    
    Parameters
    ----------
    shape : (int, int)
      The dimensions of the desired texture array.
    spacing : int
      The mean spacing between droplets.
    radius : int
      The droplet radius.
    mix : float
      The magnitude of droplet perturbations, ie the amount of 
      randomness in the positions. Droplet superposition starts
      occurring at mix=1.
      
    """
    # Create regular array of points
    nx, ny = shape
    i,j = np.meshgrid(np.arange(nx, step=spacing), np.arange(ny, step=spacing))
    
    # Randomize the position of the points. 
    i = i + np.random.randint(-spacing*mix/2., spacing*mix/2., i.shape)
    j = j + np.random.randint(-spacing*mix/2., spacing*mix/2., j.shape)
    i = np.mod(i, nx)
    j = np.mod(j, ny)
    
    # Create droplets at those points
    x = np.zeros(shape, np.float32)
    x[i, j] = 1.
    z = np.linspace(-1.5, 1.5, 2*radius+1)
    kernel = np.exp(-z**2).astype(np.float32)
    x = np.apply_along_axis(np.convolve, 0, x, kernel, 'same')
    x = np.apply_along_axis(np.convolve, 1, x, kernel, 'same')
    
    return x
